;================================
;          ZX-Assembler-2 
;      by Artic Computing LTD
;================================
;Antfarms "GUI" interface routines
;Greg Harder, 2024
;================================

ORG 43AA			;17322

@Keyboard=02BB			;Keyboard scanning subroutine
@Decode=07BD			;Keyboard decode subroutine
@Laddr=09D8			;Line address subroutine
@DFile=400C			;Address of display file
@NxtLine=4029			;Next program line to be executed
@LastK=4025			;Keyboard debounce value
@Wait=403C			;16444, Popup wait flag, IY+60
@MainForm=4089			;Stored mainform address
@FarmScr=3C00			;Stored Antfarm address
;================================
; Jump Table
;================================
	JR KeyStart		;17322, Key loop and sorter
	JR LoadScr		;17324, Load main form
	JR DropMenu		;17326, Process dropdown menu
	JR PopOK		;17328, OK Message
	JR PopTxt		;17330, Text Message
	JR SaveSCR		;17332, Save Mainform
	JR SaveFarm		;17334, Save Farm screen
;================================
; Key loop
;================================
@KeyStart
	CALL 0F2B		;SLOW mode
	CALL LoadScr		;Restore mainform
@KeyLoop
	CALL KeyScan		;Get key press
	JR Z -5			;Loop until key pressed
	LD HL,Table		;GOTO Table
;Table size
	LD BC,0018		;Size of table
	CPIR			;Locate valid keypress
	JR Z +5			;Jump if key found
	CALL Release		;Wait for key release
	JR KeyLoop		;Loop until valid key pressed
	LD C,(HL)		;Set Line Lo value
	INC HL
	LD B,(HL)		;Set Line Hi value
	RET			;On return GOTO line number
;================================
; Load Screen
;================================
@LoadScr
	CALL Release
	LD HL,MainForm		;Set source
	LD DE,(DFile)		;Set destination
@SaveIt
	LD BC,0319		;BC=byte count
	LDIR			;Copy source to desination
	RET			;Return 0
;================================
; Save Screen
;================================
@SaveScr
	LD DE,MainForm		;Set destination
	LD HL,(DFile)		;Set source
	JR SaveIt
;================================
; Save Farm
;================================
@SaveFarm
	LD DE,FarmScr		;Set destination
	JR -10
;================================
; Drop down menu
;================================
@DropMenu
	CALL Dropdown
	CALL KeyScan		;Start Keyboard loop
	JR Z -5			;Loop until keypressed
	CP "A"			;Key<"A"?
	JR C Goto10		;GOTO 10 if key<"A"
	CP "V"			;Key>="W"?
	JR NC Goto10		;GOTO 10 if key>"V"
	SUB 25 			;A=0 to 22
        LD B,00
	LD C,A 			;BC=Prog Pick
	JR Release		;Wait for key release
@Goto10
	LD HL,000A		;Locate line 10
	CALL LAddr
	LD (NxtLine),HL
	RET
;================================
; Print "Text" Popup, wait for keypress.
;================================
@PopOK
	CALL DropDown		;Print PopUp
	CALL KeyScan		;Get key press
	JR Z -5			;Loop until key pressed
	JR Goto10		;GOTO 10
;================================
; Print "Text" Popup, return 0.
;================================
@PopTxt
	CALL DropDown		;Print PopUp
	CALL Release		;Wait for key release
	LD BC,0000
	RET			;Return 0 to BASIC
;================================
; Print dropdown menu
;================================
@DropDown
	CALL LoadScr		;Restore mainform
	LD HL,(NxtLine)		;Get List addr
	LD BC,0005		;Skip REM header
	ADD HL,BC		;Start of data
	CALL Release		;Wait for key release
;Convert YXcoords to digit
	LD E,1C			;28
	LD A,(HL)		;Get Ycoord chr
	SUB E			;Digit=Chr code-28
	LD B,A			;B=Ycoord 
	INC HL			;Xcoord chr addr
	LD A,(HL)		;Get Xcoord chr
	SUB E			;Digit=Chr code-28
	LD C,A			;C=Xcoord
	INC HL			;Start of print data
	PUSH HL			;PUSH data pointer
	CALL PrntPos		;Get print position of the NW corner of dropdown
	POP BC			;POP Datapointer into BC
@DropY
	LD (DropYX+1),HL	;Save start of row addr
@DropX
	LD A,(BC)		;Get chr to print
	CP 76			;EO REM line?
	RET Z			;Return printing done
	CP 0C			;EOL marker ()?
	JR Z +3			;Jump if EOL
	LD (HL),A		;Print A
	JR +7
@DropYX
	LD HL,0000		;Get start of row addr
	ADD HL,DE		;Next row addr
	INC BC			;Increase data pointer
	JR DropY		;Start of row addr
	INC HL			;Increase print position
	INC BC			;Increase data pointer
	JR DropX
;================================
; Wait for key release
;================================
@Release
	LD A,(LastK)		;Test for keypress
	CP FF			;Keypressed?
	JR NZ Release		;Yes, loop back
	RET
;================================
; Print Position
;  B=Ycoord
;  C=Xcoord
;  BC unchange
;  On exit HL=Print Position address
;================================
@PrntPos
	PUSH BC			;PUSH YXcoords
	LD HL,0000		;Set accumulator to 0
	LD DE,0021		;Bytes/row
	LD A,B			;A=Row count
	AND A			;Row=0?
	JR Z +3			;Yes, move on
	ADD HL,DE		;HL=33*row value
	DJNZ -3			;Loop until B=0
	ADD HL,BC		;Add column offset HL=Print Position addr
	LD BC,(DFile)		;Get DFile
	INC BC			;Skip 1st N/L chr
	ADD HL,BC		;HL=print position
	POP BC			;POP YXcoords
	RET	
;================================
; Test for keypress.
; On exit...
;   A=chr code
;================================
@KeyScan
	CALL Keyboard		;Call keyboard scanning routine
	LD B,H 			;BC=Raw Key values
	LD C,L 
	LD D,C			;Test for keypress, D=255 if no key press
	INC D 			;0?
	RET Z
	CALL Decode		;Call keyboard decode routine, BC to ZX81 codes
	INC A			;Set Z flag
	LD A,(HL)
	RET			;Return, A=chr code
;================================
;Dropdown table
; Pick key values from list.
; Adjust table size in KeyStart routine.
;================================
;      Key Line
;     Code Number
;     ---- ------
;	26 044C		;A=1100
;	27 04B0		;B=1200
;	28 0514		;C=1300
;	29 0578		;D=1400
;	2A 05DC		;E=1500
;	2B 0640		;F=1600
;	2C 06A4		;G=1700
;	2D 0708		;H=1800
;	2E 076C		;I=1900
;	2F 07D0		;J=2000
;	30 O834		;K=2100, Note:CPIR will fail if "K" is used before "O" in menu
;	31 0898		;L=2200
;	32 08FC		;M=2300
;	33 0960		;N=2400
;	34 09C4		;O=2500
;	35 0A28		;P=2600
;	36 0A8C		;Q=2700
;	37 0AF0		;R=2800
;	38 0B54		;S=2900
;	39 0BB8		;T=3000
;	3A 0C1C		;U=3100
;	3B 0C80		;V=3200
;	3C 0CE4		;W=3300
;	3D 0D48		;X=3400
;	3E 0DAC		;Y=3500
;	3F 0E10		;Z=3600
;================================
; Dropdown menu table
;
; Antfarms uses B,F,H,S,W and X
; Size=8*3=24 (15H)
;
;================================
@Table
	27 04B0		;B=1200; Black Cells
	28 0514		;C=1300; Clear Quick key
	2B 0640		;F=1600; Farms
	2D 0708		;H=1800, Help
	35 0A27		;P=2600, Predefine UDGs
	38 0B54		;S=2900, Sand Cells
	3C 0CE4		;W=3300, White Cells, X=3400, eXit
	3D 0D48